unit clLazGlobals;

interface

uses
  SysUtils, Classes, DB, BufDataset, Variants;

type

  { TGlobals }

  TGlobals = class
  private
    { Private declarations }
    ClDataSet : TBufDataset;
  public
    { Public declarations }
    constructor Create;
    destructor Destroy; override;
    procedure Remove(const aVeldnaam : string);
    procedure RemoveBatch(const _namen : array of string);
    procedure Put(const aVeldnaam, aVeldwaarde : string);
    procedure PutInt(const aVeldnaam : string; const aVeldwaarde: integer);
    procedure PutBool(const aVeldnaam : string; const aVeldwaarde: boolean);
    procedure PutDate(const aVeldnaam : string; const aVeldwaarde: TDate);
    procedure RemoveLike(const _value: string);
    procedure Sum(const aVeldnaam: string; const aVeldwaarde: integer);

    function  Get(const _S1 : string; var _S2 : string) : boolean;
    function  GetVar(const _S1 : string) : string;
    function  GetVarInt(const _S1 : string) : integer;
    function  GetVarBool(const _S1 : string) : boolean;
    function  GetVarDate(const _S1 : string) : TDate;
    function  GetVarFloat(const _S1 : string) : Double;
    function  Exists(const _S1 : string) : boolean;
    function  ReadGlobal : TDataSet;
  end;

function uGlobal: TGlobals;

implementation

var
  fGlobal: TGlobals = nil;

function uGlobal: TGlobals;
begin
  if fGlobal = nil then
    fGlobal := TGlobals.Create;
  Result := fGlobal;
end;

constructor TGlobals.Create;
begin
  inherited;
    ClDataSet := TBufDataset.Create(nil);
  with ClDataSet do
  begin
    FieldDefs.Add('fieldname',ftString, 50);
    FieldDefs.Add('defvalue',ftString, 255);
  end;
  ClDataset.CreateDataset;
  ClDataSet.Active := true;
end;

destructor TGlobals.Destroy;
begin
  ClDataSet.Active := false;
  ClDataSet.free;
end;

function TGlobals.Exists(const _S1: string): boolean;
begin
   result := ClDataSet.Locate('fieldname',VarArrayOf([_S1]),[loCaseInsensitive]);
end;

function TGlobals.Get(const _S1 : string; var _S2 : string) : boolean;
begin
  with ClDataSet do
  begin
    if Locate('fieldname',VarArrayOf([_S1]),[loCaseInsensitive]) then
    begin
      _S2 := FieldByName('defvalue').AsString;
      result := true;
    end else
    begin
      _S2 := '';
      result := false;
    end;
  end;
end;

function TGlobals.GetVar(const _S1 : string) : string;
begin
  with ClDataSet do
  begin
    if Locate('fieldname',VarArrayOf([_S1]),[loCaseInsensitive]) then
      result := FieldByName('defvalue').AsString
    else result := '';
  end;
end;

function TGlobals.GetVarBool(const _S1: string): boolean;
begin
  with ClDataSet do
  begin
    if Locate('fieldname',VarArrayOf([_S1]),[loCaseInsensitive]) then
    begin
      result := FieldByName('defvalue').AsString = 'True';
    end else result := false;
  end;
end;

function TGlobals.GetVarDate(const _S1: string): TDate;
begin
  with ClDataSet do
  begin
    if Locate('fieldname',VarArrayOf([_S1]),[loCaseInsensitive]) then
      result := StrToDAte(FieldByName('defvalue').AsString)
    else result := 0;
  end;
end;

function TGlobals.GetVarFloat(const _S1: string): Double;
begin
  with ClDataSet do
  begin
    if Locate('fieldname',VarArrayOf([_S1]),[loCaseInsensitive]) then
      result := StrTofloatDef(FieldByName('defvalue').AsString,0.00)
    else result := 0.00;
  end;
end;

function TGlobals.GetVarInt(const _S1 : string) : integer;
begin
  with ClDataSet do
  begin
    if Locate('fieldname',VarArrayOf([_S1]),[loCaseInsensitive]) then
      result := StrToInt(FieldByName('defvalue').AsString)
    else result := 0;
  end;
end;

procedure TGlobals.Put(const aVeldnaam, aVeldwaarde : string);
begin
  with ClDataSet do
  begin
   if Locate('fieldname',VarArrayOf([aVeldnaam]),[loCaseInsensitive]) then
   begin
     Edit;
     FieldByName('defvalue').AsString := aVeldwaarde;
     Post;
   end else
   begin
     AppendRecord([aVeldnaam,aVeldwaarde]);
   end;
 end;
end;

procedure TGlobals.Sum(const aVeldnaam : string; const aVeldwaarde : integer);
var Somwaarde : integer;
begin
  if Exists(aVeldnaam) then
  begin
    Somwaarde := GetVarInt(aVeldnaam) + aVeldwaarde;
    if Somwaarde < 0 then SomWaarde := 0;
    ClDataSet.Edit;
    ClDataSet.FieldByName('defvalue').AsString := IntToStr(Somwaarde);
    ClDataSet.Post;
  end else
  begin
    Somwaarde := 0 + aVeldwaarde;
    ClDataSet.AppendRecord([aVeldnaam,Somwaarde]);
  end;
end;


procedure TGlobals.PutInt(const aVeldnaam : string; const aVeldwaarde: integer);
begin
  Put(aVeldnaam, IntToStr(aVeldwaarde));
end;

procedure TGlobals.PutBool(const aVeldnaam : string; const aVeldwaarde: boolean);
var Waarde : string;
begin
  if aVeldwaarde then
    Waarde := 'True'
  else
    Waarde := 'False';
  Put(aVeldnaam, waarde);
end;


procedure TGlobals.PutDate(const aVeldnaam: string; const aVeldwaarde: TDate);
begin
  Put(aVeldnaam, DateToStr(aVeldwaarde));
end;

function TGlobals.ReadGlobal : TDataSet;
begin
  result := ClDataSet;
end;

procedure TGlobals.Remove(const aVeldnaam: string);
var Ts     : TStrings;
    Teller : smallint;
begin
  Ts := TStringlist.Create;
  try
    Ts.CommaText := aVeldnaam;
    for Teller := 0 to Ts.Count - 1 do
    begin
      with ClDataSet do
      begin
        if Locate('fieldname',VarArrayOf([Ts[teller]]),[loCaseInsensitive]) then
          delete;
      end;
    end;
  finally
    Ts.Free;
  end;
end;

procedure TGlobals.RemoveBatch(const _namen: array of string);
var teller : smallint;
begin
  for teller := 0 to high(_namen) do
    remove(_namen[teller])
end;

procedure TGlobals.RemoveLike(const _value: string);
var lengte      : smallint;
    StringRange : string;
begin
  ClDataSet.First;
  Lengte := Length(_value);
  StringRange := '';
  while not ClDataset.eof do
  begin
    if copy(ClDataset.Fieldbyname('fieldname').AsString,1,Lengte) = _value then
      StringRange := StringRange + ClDataset.Fieldbyname('fieldname').AsString + ',';
    ClDataset.Next;
  end;
  Delete(StringRange,Length(StringRange) - 1,1);
  remove(StringRange)
end;

initialization

finalization
  fGlobal.Free;

end.
