unit DBase;

interface

uses
  SysUtils, Classes, variants, sqldb, dConnections, db, IBConnection, dialogs,
  clLazglobals;

type

 { TDMBase }

  TDMBase = class(TDataModule)
    LinkEvents: TSQLQuery;
    procedure DBSort(const aFieldname: string; aDataset: TDataset);
    procedure LinkEventsAfterCancel(DataSet: TDataSet);
    procedure LinkEventsAfterDelete(DataSet: TDataSet);
    procedure LinkEventsAfterOpen(DataSet: TDataSet);
    procedure LinkEventsAfterPost(DataSet: TDataSet);
    procedure LinkEventsBeforeClose(DataSet: TDataSet);
    procedure LinkEventsBeforeOpen(DataSet: TDataSet);
    procedure LinkEventsDeleteError(DataSet: TDataSet; E: EDatabaseError;
      var DataAction: TDataAction);
    procedure LinkEventsPostError(DataSet: TDataSet; E: EDatabaseError;
      var DataAction: TDataAction);
  private
    procedure TrName( const aDataset : TDataset);
    { Private declarations }
  public
    { Public declarations }
    procedure CommitQuery(const aDataset: TDataset; aRetain: Boolean=true);

    function GetIDFromContainer(const aContainerName: string): integer;
    function DBConnection : TIBConnection;
  end;

type TDMBaseLink = class of TDMBase;

implementation

{$R *.dfm}

{ TDMBase }

procedure TDMBase.LinkEventsAfterCancel(DataSet: TDataSet);
var MyQuery : TSQLQuery;
begin
  MyQuery := TSQLQuery(dataset);
  MyQuery.ApplyUpdates;
  CommitQuery(DataSet)
end;

procedure TDMBase.LinkEventsAfterDelete(DataSet: TDataSet);
var MyQuery : TSQLQuery;
begin
  try
    MyQuery := TSQLQuery(dataset);
    MyQuery.ApplyUpdates;
    CommitQuery(DataSet)
  except
    showmessage('fout tijdens delete')
  end;
end;

procedure TDMBase.LinkEventsAfterOpen(DataSet: TDataSet);
begin
  dataset.FieldByName('ID').Required := false;
//  TrName(DataSet);
end;

procedure TDMBase.LinkEventsAfterPost(DataSet: TDataSet);
var MyQuery : TSQLQuery;
begin
  MyQuery := TSQLQuery(dataset);
  MyQuery.ApplyUpdates;
  CommitQuery(DataSet)
end;

procedure TDMBase.LinkEventsBeforeClose(DataSet: TDataSet);
var MyQuery : TSQLQuery;
begin
  MyQuery := TSQLQuery(dataset);
  if MyQuery.State in [dsInsert,dsEdit] then
    Myquery.Cancel
end;

procedure TDMBase.LinkEventsBeforeOpen(DataSet: TDataSet);
var MyQuery : TSQLQuery;
begin
  MyQuery := TSQLQuery(dataset);
  TSQLtransaction(MyQuery.Transaction).DataBase := DMConnections.DBConnection;
  MyQuery.database := DMConnections.DBConnection;
end;

procedure TDMBase.LinkEventsDeleteError(DataSet: TDataSet; E: EDatabaseError;
  var DataAction: TDataAction);
begin
  showmessage(' delete' );
  dataaction := daAbort;
end;

procedure TDMBase.LinkEventsPostError(DataSet: TDataSet; E: EDatabaseError;
  var DataAction: TDataAction);
begin
  showmessage(E.Message);
  DataAction := daAbort;
end;

procedure TDMBase.TrName(const aDataset:TDataset); inline;
begin
  if (aDataset is TSQLQuery) and (TSQLQuery(aDataset).Transaction is TSQLTransaction) then begin
      showmessage(TSQLtransaction(TsqlQuery(aDataset).Transaction).Name);
  end;
end;



procedure TDMBase.CommitQuery(const aDataset:TDataset; aRetain :Boolean = true); inline;
begin
  if (aDataset is TSQLQuery) and (TSQLQuery(aDataset).Transaction is TSQLTransaction) then begin
    if aRetain then
      TSQLtransaction(TsqlQuery(aDataset).Transaction).CommitRetaining
    else
      TSQLtransaction(TsqlQuery(aDataset).Transaction).Commit;
  end;
end;

function TDMBase.GetIDFromContainer(const aContainerName : string) : integer;
var MyQuery : TSQLQuery;
    SQLTransaction : TSQLTransaction;
begin
  MyQuery := TSQLQuery.create(nil);
  SQLTransaction := TSQLTransaction.create(nil);
  try
    try
      MyQuery.DataBase        := DMConnections.DBConnection;
      MyQuery.Transaction     := SQLTransaction;
      SQLTransaction.DataBase := DMConnections.DBConnection;
      with MyQuery do
      begin
        SQL.Text := format('select gen_id(%s, 1) from rdb$database',[aContainerName]);
        Open;
        // Get the ID of the new record
        result := FieldByName('GEN_ID').AsInteger;
        Close;
      end;
    except
      result := 0;
    end;
  finally
    MyQuery.free;
    SQLTransaction.free;
  end;

end;

function TDMBase.DBConnection: TIBConnection;
begin
  result := DMConnections.IBConnection1;
end;


procedure TDMBase.DBSort(const aFieldname: string; aDataset : TDataset);
var Query : TSQLQuery;
begin
  if aDataset.state in [dsBrowse] then
  begin
    Query := adataset as TSQLQuery;
    Query.indexfieldnames := aFieldName;
  end;
end;


end.
